package com.stpos.a8pos.devicesdk.impl.nexgo

import com.nexgo.oaf.apiv3.DeviceEngine
import com.nexgo.oaf.apiv3.device.led.LightModeEnum
import com.stpos.a8pos.devicesdk.Led


class NexgoLed(engine: DeviceEngine) : Led {

    private val led = engine.ledDriver


    override fun turnOn(vararg lights: Int) {
        setLeds(true, *lights)
    }

    override fun turnOff(vararg lights: Int) {
        setLeds(false, *lights)
    }

    private fun setLeds(enable: Boolean, vararg lights: Int) {
        lights.forEach {
            val color = it.mapToLightModeEnum()

            if (color != null) {
                led.setLed(color, enable)
            }
        }
    }
}

/**
 * Tries to map an integer (containing an ordinal of available LED colors) to
 * Nexgo's internal LED color representation.
 */
private fun Int.mapToLightModeEnum() = when (this) {
    Led.RED -> LightModeEnum.RED
    Led.GREEN -> LightModeEnum.GREEN
    Led.BLUE -> LightModeEnum.BLUE
    else -> null
}