/* **************************************************************************
 * Copyright (C) 2020 Solitea Pay s.r.o. - All Rights Reserved
 *
 * This file is part of STPAY
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 *
 * Written by Pavel Kocábek, pavel.kocabek@smart-software.cz, 08.12.2020
 *
 * *************************************************************************/

package com.stpos.a8pos.devicesdk.impl.nexgo

import com.nexgo.oaf.apiv3.DeviceEngine
import com.nexgo.oaf.apiv3.SdkResult
import com.nexgo.oaf.apiv3.device.pinpad.AlgorithmModeEnum
import com.nexgo.oaf.apiv3.device.pinpad.CipherModeEnum
import com.nexgo.oaf.apiv3.device.pinpad.WorkKeyTypeEnum
import com.stpos.a8pos.config.KeyIds
import com.stpos.a8pos.devicesdk.DeviceException
import com.stpos.a8pos.devicesdk.emv.KAPId
import com.stpos.a8pos.devicesdk.pinpad.*
import com.stpos.a8pos.devicesdk.pinpad.CipherSettings.BlockCipherMode.*
import io.reactivex.Completable
import io.reactivex.Observable
import timber.log.Timber

class NexgoRxPinPad(engine: DeviceEngine) : RxPinPad {
    private val pinPad = engine.pinPad

    override fun open() {
        // no action required
    }

    override fun close() {
        // no action required
    }

    override fun getAccessableKapIds(expectedKapsNum: Int): List<KAPId>? {
        TODO("Not yet implemented (should be unused)")
    }

    override fun getKapMode(): Int {
        TODO("Not yet implemented (should be unused)")
    }

    override fun downloadMkeyFromMasterPos(): Completable {
        TODO("Not yet implemented (should be unused)")
    }

    override fun cancelDownloadMkeyFromMasterPos() {
        TODO("Not yet implemented (should be unused)")
    }

    override fun inputPinOffline(pinLimit: ByteArray): Observable<RxPinPad.PinPadEvent> {
        TODO("Not yet implemented")
    }

    override fun inputPinOffline(timeout: Int, betweenPinKeyTimeout: Int, pinLimit: ByteArray, pinBlockFormat: Byte?): Observable<RxPinPad.PinPadEvent> {
        TODO("Not yet implemented")
    }

    override fun inputPinOnline(pinKeyIndex: Int, pan: String, pinLimit: ByteArray): Observable<RxPinPad.PinPadEvent> {
        TODO("Not yet implemented")
    }

    override fun inputPinOnline(keyHandle: KeyHandle?, pan: String, timeout: Int, betweenPinKeyTimeout: Int, pinLimit: ByteArray, pinBlockFormat: Byte?): Observable<RxPinPad.PinPadEvent> {
        TODO("Not yet implemented")
    }

    override fun inputPinOnline(pinKeyIndex: Int, pan: String, timeout: Int, betweenPinKeyTimeout: Int, pinLimit: ByteArray, pinBlockFormat: Byte?): Observable<RxPinPad.PinPadEvent> {
        TODO("Not yet implemented")
    }

    override fun cancelInputPin(reason: Int) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun initDukptKsn(keyId: Int, ksnData: ByteArray?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun increaseDukptKsn(keyId: Int) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun getDukptCurrentKsn(keyId: Int): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun setKeyAlgorithm(keyAlgorithm: Int) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun setEncKeyFormat(keyFormat: Int) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun isKeyExist(keyId: Int) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun isKeyExist(keyHandle: KeyHandle?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun doKeyExist(keyId: Int): Boolean {
        TODO("Not yet implemented (should be unused)")
    }

    override fun loadMasterKey(keyId: Int, key: ByteArray?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun loadPinKey(mainKeyId: Int, pinKeyId: Int, encKey: ByteArray?, checkValue: ByteArray?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun loadMacKey(mainKeyId: Int, macKeyId: Int, encKey: ByteArray?, checkValue: ByteArray?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun loadTrackKey(mainKeyId: Int, trackKeyId: Int, encKey: ByteArray?, checkValue: ByteArray?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun loadEncKey(keyType: Int, mainKeyId: Int, keyId: Int, encKey: ByteArray?, checkValue: ByteArray?) {
        if (keyId == KeyIds.pinWorkingKeyIndex.get()) {
            // map pinWorkingKeyIndex keys to Nexgo PIN key enum type
            val res = pinPad.writeWKey(mainKeyId, WorkKeyTypeEnum.PINKEY, encKey, encKey!!.size)
            if (res != SdkResult.Success) {
                throw DeviceException("Nexgo SDK returned $res")
            }
        } else {
            Timber.w("loadEncLKey called, but with an unsupported keyId")
        }
    }

    override fun loadEncKey(srcKeyHandle: KeyHandle?, dstKeyHandle: KeyHandle?, encKeyFmt: Int, keyCfg: KeyCfg?, encKey: ByteArray?, checkValue: ByteArray?) {
        TODO("Not yet implemented (unused?)")
    }

    override fun loadPlainKey(keyType: Int, keyId: Int, key: ByteArray?) {
        TODO("Not yet implemented (unused?)")
    }

    override fun deleteKey(keyId: Int) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun deleteKey(keyHandle: KeyHandle?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun calculateKCV(keyId: Int): ByteArray =
            if (keyId < 3)
                // indices 0, 1, 2 refer to master keys
                pinPad.encryptByMKey(keyId, ByteArray(8), 8).copyOfRange(0, 2)
            else
                when (keyId) {
                    // indices 8, 9 refer to nexgo's working keys (ideally, KCV calc. methods should be separated for master/worker keys,.
                        // but the current interface does not support it, see Landi implementation)
                    KeyIds.pinWorkingKeyIndex.get() -> pinPad.calcWKeyKCV(0, WorkKeyTypeEnum.PINKEY)
                    KeyIds.macWorkingKeyIndex.get() -> pinPad.calcWKeyKCV(0, WorkKeyTypeEnum.MACKEY)
                    else -> TODO("Not implemented")
                }


    override fun calculateKCV(keyHandle: KeyHandle?): ByteArray = keyHandle?.let {
        calculateKCV(it.keyId)
    } ?: ByteArray(3)

    override fun calculateMAC(algorithm: Int, paddingMode: Int, keyId: Int, data: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun calculateMAC(algorithm: Int, paddingMode: Int, keyId: Int, icvData: ByteArray?, data: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun calculateMAC(algorithm: Int, paddingMode: Int, keyHandle: KeyHandle?, icvData: ByteArray?, data: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun encryptMagTrack(mode: Int, keyId: Int, trkData: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun encryptMagTrack(mode: Int, keyHandle: KeyHandle?, trkData: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun calculateDES(setEncOrDec: Int, keyId: Int, data: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun calculateDES(settings: CipherSettings, keyId: Int, data: ByteArray): ByteArray {
        pinPad.setAlgorithmMode(AlgorithmModeEnum.DES)
        pinPad.setCipherMode(
            when (settings.blockCipherMode) {
                CBC -> CipherModeEnum.CBC
                ECB -> CipherModeEnum.ECB
            }
        )
        return pinPad.encryptByMKey(keyId, data, data.size)
    }

    override fun calculateDES(keyHandle: KeyHandle?, desMode: CipherSettings?, icvData: ByteArray?, data: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun getRandom(length: Int): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun verifyOfflinePin(offlinePinVerify: OfflinePinVerify?, pinPublicKey: PinPublicKey?, pinVerifyResult: PinVerifyResult?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun setPinPadSN(serialNum: ByteArray?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun formatPinPad() {
        TODO("Not yet implemented (should be unused)")
    }

    override fun switchPinPadToWorkMode() {
        TODO("Not yet implemented (should be unused)")
    }

    override fun getLastErrorMessage(): String {
        TODO("Not yet implemented (should be unused)")
    }

    override fun beep(beepMs: Int): Boolean {
        TODO("Not yet implemented (should be unused)")
    }

    override fun reset(): Boolean {
        TODO("Not yet implemented (should be unused)")
    }

    override fun initRemoteKeyLoadProc(mode: Int, reqInfo: ReqInfo?): Int {
        TODO("Not yet implemented (should be unused)")
    }

    override fun authKMSCrt(mode: Int, authInfo: AuthInfo?, authOut: AuthOut?): Int {
        TODO("Not yet implemented (should be unused)")
    }

    override fun checkCIDandSignData(mode: Int, cid: ByteArray?, signData: ByteArray?, signOut: SignOut?): Int {
        TODO("Not yet implemented (should be unused)")
    }

    override fun signData(mode: Int, keyHandle: KeyHandle?, inData: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun loadRKISv2RemoteKey(mode: Int, keyIndex: Int, key: ByteArray?): Int {
        TODO("Not yet implemented (should be unused)")
    }

    override fun loadRKISv2RemoteKey(mode: Int, keyHandle: KeyHandle?, key: ByteArray?): Int {
        TODO("Not yet implemented (should be unused)")
    }

    override fun display(line: Int, message: String?): Boolean {
        TODO("Not yet implemented (should be unused)")
    }

    override fun setPinpadConfig(pinpadCfg: PinpadCfg?) {
        TODO("Not yet implemented (should be unused)")
    }

    override fun remoteInjectTMK(RKISServerAddress: String?, RKISServerPort: Int, terminalId: String?, merchantId: String?, masterKeyIndex: Int): Observable<String> {
        TODO("Not yet implemented (should be unused)")
    }

    override fun remoteInjectTMK(RKISServerAddress: String?, RKISServerPort: Int, terminalId: String?, merchantId: String?, masterKeyIndex: Int, tpdu: String?, kmsID: String?): Observable<String> {
        TODO("Not yet implemented (should be unused)")
    }

    override fun getTUSN(mode: Int, input: ByteArray?): TusnData {
        TODO("Not yet implemented (should be unused)")
    }

    override fun isEppConnected(): Boolean {
        TODO("Not yet implemented (should be unused)")
    }

    override fun calculateHMAC(keyHandle: KeyHandle?, hashAlgorithm: Int, macKeyType: Int, data: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun generateKey(mode: Char, srcKeyId: Int, dstKeyId: Int, keyCfg: KeyCfg?, inData: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }

    override fun generateKey(mode: Char, srcKeyHandle: KeyHandle?, dstKeyHandle: KeyHandle?, keyCfg: KeyCfg?, inData: ByteArray?): ByteArray {
        TODO("Not yet implemented (should be unused)")
    }
}