package com.stpos.a8pos.devicesdk.impl.nexgo

import android.content.Context
import com.nexgo.oaf.apiv3.APIProxy
import com.stpos.a8pos.devicesdk.BaseDeviceSdk
import com.stpos.a8pos.devicesdk.card.RFDevice
import com.stpos.a8pos.devicesdk.emv.BaseParameter
import com.stpos.a8pos.devicesdk.emv.EmvParameters
import com.stpos.a8pos.devicesdk.emv.MasterParameter
import com.stpos.a8pos.devicesdk.emv.PbocParameter
import com.stpos.a8pos.devicesdk.impl.landi.emv.LandiMockEmvData
import com.stpos.a8pos.devicesdk.impl.nexgo.card.NexgoContactIcCardReader
import com.stpos.a8pos.devicesdk.impl.nexgo.card.NexgoContactlessReader
import com.stpos.a8pos.devicesdk.impl.nexgo.card.NexgoMagCardReader
import com.stpos.a8pos.devicesdk.impl.nexgo.emv.NexgoEmv
import com.stpos.a8pos.devicesdk.pinpad.RxPinPad
import com.stpos.a8pos.devicesdk.printer.Format
import com.stpos.a8pos.devicesdk.printer.Printer
import com.stpos.a8pos.devicesdk.tms.Tms
import io.reactivex.Completable
import usdk.printer.VectorFormat
import usdk.printer.line.Line

const val N86_MODEL_NAME = "N86"

class NexgoSdk(context: Context) : BaseDeviceSdk(context) {

    private val engine = APIProxy.getDeviceEngine(context)

    override fun isConnected(): Boolean = true

    override fun connect(): Completable = Completable.complete()

    override fun disconnect() { }

    override fun getEmv() = NexgoEmv(engine)

    override fun getBeeper() = NexgoBeeper(engine)

    override fun supportsMagneticCard(): Boolean {
        TODO("Not yet implemented")
    }

    override fun supportsIcCard(): Boolean {
        TODO("Not yet implemented")
    }

    override fun getMagCardReader() = NexgoMagCardReader(engine)

    override fun getContactlessCardReader(rfDevice: RFDevice) = NexgoContactlessReader(engine)

    override fun getContactIcCardReader() = NexgoContactIcCardReader(engine)

    override fun getLed() = NexgoLed(engine)

    override fun getEmvData() = com.stpos.a8pos.devicesdk.impl.mock.emv.MockEmvData()

    override fun createPinPad(): RxPinPad = NexgoRxPinPad(engine)

    override fun createEmvParameters() = EmvParameters()

    override fun createBaseParameter(): BaseParameter {
        TODO("Not yet implemented")
    }

    override fun createPbocParameter(): PbocParameter {
        TODO("Not yet implemented")
    }

    override fun createMasterParameter(): MasterParameter {
        TODO("Not yet implemented")
    }

    override fun getMockEmvData() = LandiMockEmvData()

    override fun getPrinter(): Printer = object : Printer {
        override fun getPaperWidth(): Int {
            return 0
        }

        override fun initWebView(context: Context?, width: Int) {
        }

        override fun releaseWebView(context: Context?) {
        }

        override fun getErrorMessage(errorCode: Int): String? {
            return ""
        }

        override fun getStatus(): Int {
            return 0
        }

        override fun setGray(gray: Int) {
        }

        override fun addText(text: String?) {
        }

        override fun addText(format: Format, text: String?) {
        }

        override fun addVectorText(text: String?) {
        }

        override fun addVectorText(left: String?, right: String?) {
        }

        override fun addVectorText(left: String?, leftFormat: VectorFormat, right: String?, rightFormat: VectorFormat) {
        }

        override fun addVectorText(text: String?, format: VectorFormat?) {
        }

        override fun addVectorTexts(texts: List<String?>): Completable {
            return Completable.complete()
        }

        override fun addVectorLines(lines: List<Line?>?): Completable {
            return Completable.complete()
        }

        override fun getBitmapBytesFromPrintLines(lines: List<Line?>?): ByteArray? {
            return ByteArray(1)
        }

        override fun addVectorDivision(thickness: Int, height: Int) {
        }

        override fun addVectorDivision(thickness: Int) {
        }

        override fun addVectorDivision() {
        }

        override fun addMixText(align: Int, textBlockList: List<Format>) {
        }

        override fun addMixStyleText(align: Int, textBlockList: List<Format>) {
        }

        override fun addBarCode(format: Format, barCode: String?) {
        }

        override fun addQrCode(format: Format, imageHeight: Int, qrCode: String?) {
        }

        override fun addBmpImage(image: ByteArray) {
        }

        override fun addBmpImage(offset: Int, factor: Int, image: ByteArray) {
        }

        override fun addImage(format: Format, image: ByteArray) {
        }

        override fun addImage(image: ByteArray) {
        }

        override fun addHtml(templateId: String?, json: String?): Completable {
            return Completable.complete()
        }

        override fun addPDF(filePath: String?): Completable {
            return Completable.complete()
        }

        override fun addImages(images: List<ByteArray>) {
        }

        override fun setLineSpace(space: Int) {

        }

        override fun setIndexPage(indexUrl: String?) {

        }

        override fun feedLine(lines: Int) {

        }

        override fun feedPixel(pix: Int) {

        }

        override fun print(): Completable {
            return Completable.complete()
        }

        override fun cutPaper(autoFeedPaper: Boolean) {

        }

        override fun setPrintFormat(format: Int, value: Int) {
        }

        override fun setXSpace(xSpace: Int) {
        }

        override fun setYSpace(ySpace: Int) {

        }

    }

    override fun getDeviceManager() = NexgoDeviceManager(engine)

    override fun getTms(): Tms {
        TODO("Not yet implemented")
    }

    override fun setupPinPadOrder() {
        // TODO implement
    }

    override fun setDebugLogLevel() {
        // TODO implement
    }

    override fun updateKernel() = Completable.complete()
}