package com.stpos.a8pos.devicesdk.impl.nexgo.emv

import com.nexgo.common.ByteUtils
import com.nexgo.oaf.apiv3.emv.AidEntity
import com.nexgo.oaf.apiv3.emv.CapkEntity
import com.stpos.a8pos.devicesdk.emv.CAPublicKey
import com.stpos.a8pos.devicesdk.emv.EmvParameters

/**
 * Converts properties of a supported AID from application specific class instance to a Nexgo class instance
 */
fun convertAid(emvParameters: EmvParameters, aid: String): AidEntity = AidEntity().apply {
    ddol = ByteUtils.byteArray2HexString(emvParameters.ddol)
    this.aid = aid
    appVerNum = ByteUtils.byteArray2HexString(emvParameters.applicationVersion)
    asi = 0
    contactlessTransLimit = 999999999L
    contactlessCvmLimit = 50000L
    contactlessFloorLimit = 0
    floorLimit = 0
    maxTargetPercent = 90
    onlinePinCap = 1
    tacDefault = ByteUtils.byteArray2HexString(emvParameters.defaultTac)
    tacDenial = ByteUtils.byteArray2HexString(emvParameters.denialTac)
    tacOnline = ByteUtils.byteArray2HexString(emvParameters.onlineTac)
    targetPercent = emvParameters.randomSelectionPercentage?.toInt()!!
    threshold = emvParameters.randomSelectionThreshold!!
    transLimit = 999999999L
}

/**
 * Converts a supported CA public key from application specific class instance to a Nexgo class instance
 */
fun convertPublicKey(caPublicKey: CAPublicKey) = CapkEntity().apply {
    arithInd = 1
    capkIdx = caPublicKey.index.toUByte().toInt()
    checkSum = ByteUtils.byteArray2HexString(caPublicKey.hash)
    expireDate = ByteUtils.byteArray2HexString(caPublicKey.expDate)
    exponent = ByteUtils.byteArray2HexString(caPublicKey.exp)
    hashInd = 1
    modulus = ByteUtils.byteArray2HexString(caPublicKey.mod)
    rid = ByteUtils.byteArray2HexString(caPublicKey.rid)
}