package com.stpos.a8pos.devicesdk.impl.nexgo.emv

import com.nexgo.common.ByteUtils
import com.nexgo.oaf.apiv3.emv.EmvHandler
import com.stpos.a8pos.devicesdk.impl.nexgo.util.nexgoDebugLog
import com.stpos.a8pos.emv.EmvDataConfig
import com.stpos.a8pos.sale.StaticEmvData

typealias CapkListAndAidParaInitResult = Pair<Int, Int>

/**
 * Initializes the the given [EmvHandler] with the list of supported AIDs and card issuer public keys
 */
interface NexgoEmvInitializer {
    fun initializeEmv(emvHandler: EmvHandler): CapkListAndAidParaInitResult
}

object NexgoEmvInitializerImpl : NexgoEmvInitializer {
    override fun initializeEmv(emvHandler: EmvHandler): CapkListAndAidParaInitResult {
        emvHandler.delAllAid()
        emvHandler.delAllCapk()
        emvHandler.emvDebugLog(true)
        val capkListInitResult = emvHandler.setCAPKList(
            // use list of public keys already present in the application
            EmvDataConfig.getPublicKeys()
                .filter {
                    // keep only keys that have all attributes set
                    it.rid != null &&
                            it.index != null &&
                            it.mod != null &&
                            it.exp != null &&
                            it.hash != null &&
                            it.expDate != null
                }
                .filter {
                    // following CAPK can't be loaded on Nexgo
                    it.hash?.let { hash ->
                        ByteUtils.byteArray2HexString(hash) != "FBDDBD70B3C681D3740CB4E9B8763420ACD86A"
                    } ?: true
                }
                .map {
                    convertPublicKey(it)
                }
        )
        val aidParaListInitResult = emvHandler.setAidParaList(
            // use the list of supported AIDs already present in the application
            StaticEmvData.getEmvParameters().map { convertAid(it.value, it.key) }
        )
        nexgoDebugLog("Nexgo AID list:")
        emvHandler.aidList.forEach {
            nexgoDebugLog("${it.aid} - ${it.contactlessCvmLimit}")
        }
        return Pair(capkListInitResult, aidParaListInitResult)
    }
}